#include "defs.h"

/*
 *  Trap the keyboard interrupt signal to force a disconnect. <LH:881125:v2.0>
 */

void
interrupt ()	/*  program interrupted by signal - disconnect */
{
	disconnect (I_DISCONNECTED);
}


/*
 *  Tell other side to disconnect, kill the keyboard-read process, clean up
 *  windows and then exit. <LH:881125:v2.0>
 */

void
disconnect (how)
int how;
{
	static beenhere = FALSE;

	if (beenhere++)
		return;
		
	alarm (0);
	signal (SIGALRM, SIG_IGN);
	signal (SIGINT, SIG_IGN);
	signal (SIGHUP, SIG_IGN);
	signal (SIGQUIT, SIG_IGN);
	
	if (readpid)	/* kill keyboard-read process */
	{
		kill (readpid, 9);
		wait ((int *)0);
	}

	sendmsg (hispid, DISCONNECT);

	switch (how)
	{
		case I_DISCONNECTED:
			waddstr (mywin, "\n[You have disconnected]");
			break;

		case HE_DISCONNECTED:
			waddstr (mywin, "\n[Your party has disconnected]");
			beep ();
			break;

		case PROGRAM_ERROR:
			waddstr (mywin, "\n[Program failure - disconnected]");
			break;
			
		case NO_ANSWER:
			wheader (mywin, "[Your party would not respond]\n");
			break;

		case NOT_LOGGED_IN:
			sprintf (tmp, "[Your party is not logged on %]\n",
								histty);
			wheader (tmp);
			break;

		case TTY_DISABLED:
			wheader (mywin, "[Your party has disabled messages]\n");
			break;
	}

        wnoutrefresh (mywin);
        wnoutrefresh (hiswin);
        wnoutrefresh (dividewin);
	doupdate ();
	endwin ();

	msgctl (msgid, IPC_RMID, 0);
	unlink (tempfile);		/* should have been done before */
	printf ("\n");


	if (how == PROGRAM_ERROR)
		exit (4);

	if (!connected)
		notify (DISCONNECT);

	exit(0);
}
