#include "defs.h"

/*
 *	Print error message and disconnect.
 */

void
error (message, arg1, arg2)
char *message, *arg1, *arg2;
{
		static char beenhere = FALSE;
		
		if (beenhere++)
			return;

		wmove (hiswin, HIS_WINLINES - 2, 0);
		wprintw (hiswin, "%s: ", progname);
		wprintw (hiswin, message, arg1, arg2);
		wrefresh (hiswin);
		disconnect (PROGRAM_ERROR);
}


/*
 *  Read from message queue, any message dedicated to this process.
 *
 *  The returned message does also include information about which
 *  process it was sent from.
 *
 *  <LH:881125:v2.0>
 */


MSG *
getmsg (pid)
int pid;
{
	static MSG msg;
	register MSG *msgp = &msg;
	
	if (msgrcv (msgid, msgp, MSGSIZE, (long)pid, 0) == -1)
	{
		switch (errno)
		{
			case EINTR:		/* interrupted */
				return (NULL);
		
			case EIDRM:		/* msgid removed */
				disconnect (HE_DISCONNECTED);
				return (NULL);
	
			default:		/* other problems */
				error ("msgrcv");
				return (NULL);
		}
	}
	return (msgp);
}


/*
 * Send a character or command to a specific process.
 *
 *  <LH:881125:v2.0>
 */

void
sendmsg (pid, c)
int pid;
int c;
{
	static MSG msg;
	register MSG *msgp = &msg;

	msgp->to_pid = pid;
	msgp->from_pid = mypid;
	msgp->c = c;
	
	while (msgsnd (msgid, msgp, MSGSIZE, 0) == -1)
	{
		switch (errno)
		{
			case EINTR:		/* interrupted */
				continue;
		
			case EIDRM:		/* msgid removed */
			default:
				disconnect (HE_DISCONNECTED);
				return;
		}
	}
}

/*
 *  Determine whether file is not in use.
 */

out_of_date(fname)
char	*fname;
{
	struct stat stat_buf;

	if (stat (fname, &stat_buf) == -1)	/*  assume doesn't exist  */
		return (FALSE);

	if ((time ((long *) 0) - stat_buf.st_mtime) > (MAXTRY * TRY_TIME))
		return (TRUE);
		
	return (FALSE);
}

/*
 *  Get full name of user from the gecos field of the passwd file.
 *
 *  <LH:881125:v2.0>
 */

char *
fullname (logname)
char	*logname;
{
	register struct passwd *p;
	
	return ((p = getpwnam (logname)) ?  p->pw_gecos : "no user");
}

